from beartype.typing import (
    Any, Iterable, Callable, List, Mapping, Optional, TypeVar, Type, Tuple,
    Union
)

assert Any
assert Iterable
assert Callable
assert List
assert Mapping
assert Optional
assert TypeVar
assert Type
assert Tuple
assert Union
